/*
 * Copyright 2012, 2017 INFN
 *
 * Licensed under the EUPL, Version 1.2 or – as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 * https://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 */

/**
 * WARNING(schwerin): Use with extreme caution.  Prefer the AtomicWord<> types from atomic_word.h.
 *
 * The atomic_intrinsics module provides low-level atomic operations for manipulating memory.
 * Implementations are platform specific, so this file describes the interface and includes
 * the appropriate os/compiler-specific headers.
 *
 * For supported word types, the atomic_intrinsics headers provide implementations of template
 * classes of the following form:
 *
 * template <typename T> class AtomicIntrinsics {
 *     static T load(volatile const T* value);
 *     static T store(volatile T* dest, T newValue);
 *     static T compareAndSwap(volatile T* dest, T expected, T newValue);
 *     static T swap(volatile T* dest, T newValue);
 *     static T fetchAndAdd(volatile T* dest, T increment);
 * };
 *
 * All of the functions assume that the volatile T pointers are naturally aligned, and may not
 * operate as expected, if they are not so aligned.
 *
 * The behavior of the functions is analogous to the same-named member functions of the AtomicWord
 * template type in atomic_word.h.
 */

#pragma once

#if defined(_WIN32)
#include <chaos/common/bson/platform/atomic_intrinsics_win32.h>
#elif defined(__GNUC__)
#if defined(__i386__) || defined(__x86_64__)
#include <chaos/common/bson/platform/atomic_intrinsics_gcc_intel.h>
#else
#include <chaos/common/bson/platform/atomic_intrinsics_gcc_generic.h>
#endif
#else
#error "Unsupported os/compiler family"
#endif
